from string import Template
from enocean_webthing.app import App
from enocean_webthing.enocean_webthing import run_server

PACKAGENAME = 'enocean_webthing'
ENTRY_POINT = "enocean"
DESCRIPTION = "A web connected enocean gateway"


UNIT_TEMPLATE = Template('''
[Unit]
Description=$packagename
After=syslog.target

[Service]
Type=simple
ExecStart=$entrypoint --command listen --verbose $verbose --port $port --path $path --devices '$devices'  
SyslogIdentifier=$packagename
StandardOutput=syslog
StandardError=syslog
Restart=always
RestartSec=3

[Install]
WantedBy=multi-user.target
''')



class GatewayApp(App):


    def do_process_command(self, command:str, port: int, verbose: bool, args) -> bool:
        if command == 'listen':
            print("running " + self.packagename + " on port " + str(port))
            addresses = [address.strip() for address in args.devices.split(",")]
            run_server(port, self.description, args.path, addresses)
            return True
        elif args.command == 'register':
            print("register " + self.packagename + " on port " + str(port) + " and starting it")
            unit = UNIT_TEMPLATE.substitute(packagename=self.packagename,
                                            entrypoint=self.entrypoint,
                                            port=port,
                                            verbose=verbose,
                                            path=args.path,
                                            devices=args.devices)
            self.unit.register(port, unit)
            return True
        else:
            return False


def main():
    GatewayApp(PACKAGENAME, ENTRY_POINT, DESCRIPTION).handle_command()


if __name__ == '__main__':
    main()



import logging
from threading import Thread
from typing import List
import enocean.utils
from enocean.communicators.serialcommunicator import SerialCommunicator
from enocean.protocol.constants import PACKET, RORG
import queue


logging.basicConfig(format='%(asctime)s %(name)-20s: %(levelname)-8s %(message)s', level=logging.INFO, datefmt='%Y-%m-%d %H:%M:%S')


class Device:

    def handle_packet(self, packet) -> bool:
        pass


class WindowHandle(Device):

    @staticmethod
    def supports(eep_id: str) -> bool:
        return eep_id.upper() == 'F6:10:00'

    def __init__(self, eep_id: str, enocean_id: str, callback):
        self.callback = callback
        self.sender = enocean_id.upper()
        self.sender_hex_string: List[int] = enocean.utils.from_hex_string(self.sender)
        self.eep_id = eep_id.upper()
        self.eep_id_hex_string: List[int] = enocean.utils.from_hex_string(self.eep_id)
        logging.info("window handle (eep_id: " + eep_id + ", enocean_id: " + enocean_id +")")


    def handle_packet(self, packet) -> bool:
        is_handled = False
        try:
            if self.eep_id_hex_string[0] == 0xf6 and packet.packet_type == PACKET.RADIO_ERP1 and packet.rorg == RORG.RPS:
                packet.parse_eep(self.eep_id_hex_string[1], self.eep_id_hex_string[2])
                state = packet.parsed['WIN']['raw_value']  #  WIN: {'description': 'Window handle', 'unit': '', 'value': 'Moved from vertical to down', 'raw_value': 3}
                if self.sender == packet.sender_hex:
                    self.callback(state)
                    is_handled = True
        except Exception as e:
            logging.warning("error occurred by handling packet", e)
        return is_handled

class Enocean:

    def __init__(self, port: str, devices: List[Device]):
        self.running = True
        self.devices = devices
        self.communicator = SerialCommunicator(port=port)

    def stop(self):
        self.running= False

    def receive(self, background: bool = False):
        if background:
            Thread(target=self.__receive, daemon=True).start()
        else:
            self.__receive()

    def __receive(self):
        try:
            self.communicator.start()
            if self.communicator.base_id is None:
                logging.warning('init failed')
            else:
                logging.info('The Base ID of your module is %s.' % enocean.utils.to_hex_string(self.communicator.base_id))

            # endless loop receiving radio packets
            while self.communicator.is_alive() and self.running:
                try:
                    # Loop to empty the queue...
                    packet = self.communicator.receive.get(block=True, timeout=1)
                    for device in self.devices:
                        is_handled = device.handle_packet(packet)
                        if is_handled:
                            break
                except queue.Empty:
                    continue
                except KeyboardInterrupt:
                    break
                except Exception as e:
                    logging.warning("error occurred by processing packet", e)
        finally:
            self.communicator.stop()





from setuptools import setup
from os import path

PACKAGENAME = 'enocean_webthing'
ENTRY_POINT = "enocean"
DESCRIPTION = "A web connected enocean gateway"


this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=PACKAGENAME,
    packages=[PACKAGENAME],
    version_config={
        "version_format": "{tag}.dev{sha}",
        "starting_version": "0.0.1"
    },
    setup_requires=['better-setuptools-git-version'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Gregor Roth',
    author_email='gregor.roth@web.de',
    url='https://github.com/grro/enocean_webthing',
    entry_points={
        'console_scripts': [
            ENTRY_POINT + '=' + PACKAGENAME + ':main'
        ]
    },
    keywords=[
        'webthings', 'home automation', 'enocean', 'gateway'
    ],
    install_requires=[
        'webthing==0.15.0',  'enocean==0.60.1'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
)




from time import sleep
from typing import List
from webthing import (MultipleThings, Property, Thing, Value, WebThingServer)
from enocean_webthing.enocean_ting import Enocean, WindowHandle
import logging
import tornado.ioloop


class WindowHandleWebThing(Thing):

    # regarding capabilities refer https://iot.mozilla.org/schemas
    # there is also another schema registry http://iotschema.org/docs/full.html not used by webthing

    def __init__(self, description: str, name: str, eep_id: str, enocean_id: str):
        Thing.__init__(
            self,
            'urn:dev:ops:window-handle-1',
            'WindowHandle ' + name,
            ['MultiLevelSensor'],
            description
        )

        self.ioloop = tornado.ioloop.IOLoop.current()

        self.device = WindowHandle(eep_id, enocean_id, self.on_state_updated)

        self.eepid = Value(eep_id)
        self.add_property(
            Property(self,
                     'eep_id',
                     self.eepid,
                     metadata={
                         'title': 'eep id',
                         "type": "string",
                         'description': '"The eep id',
                         'readOnly': True,
                     }))

        self.enoceanid = Value(enocean_id)
        self.add_property(
            Property(self,
                     'enocean_id',
                     self.enoceanid,
                     metadata={
                         'title': 'enocean id',
                         "type": "string",
                         'description': '"The enocean id',
                         'readOnly': True,
                     }))

        self.state = Value(3)
        self.add_property(
            Property(self,
                     'state',
                     self.state,
                     metadata={
                         'title': 'State',
                         "type": "integer",
                         'description': 'The state of the handle',
                         'readOnly': True,
                     }))

        self.state_text = Value("CLOSED")
        self.add_property(
            Property(self,
                     'state_text',
                     self.state_text,
                     metadata={
                         'title': 'State Description',
                         "type": "string",
                         'description': 'The state description',
                         'readOnly': True,
                     }))

    def on_state_updated(self, new_state: int):
        self.ioloop.add_callback(self.__update_state, new_state)

    def __update_state(self, new_state: int):
        self.state.notify_of_external_update(new_state)
        if new_state == 1:
            self.state_text.notify_of_external_update("TILTED")
        elif new_state == 2:
            self.state_text.notify_of_external_update("OPEN")
        else:
            self.state_text.notify_of_external_update("CLOSED")


def run_server(port: int, description: str, path: str, addresses: List[str]):
    enocean_webthings = []
    for address in sorted(addresses):
        name, eep_id, enocean_id = address.split("/")
        if WindowHandle.supports(eep_id):
            enocean_webthings.append(WindowHandleWebThing(description, name, eep_id, enocean_id))
        else:
            logging.warning("unsupported device (eep_id: " + eep_id + ", enocean_id: " + enocean_id +"). Ignoreing it")

    enocean = Enocean(path, [enocean_webthing.device for enocean_webthing in enocean_webthings])
    server = WebThingServer(MultipleThings(enocean_webthings, 'devices'), port=port, disable_host_validation=True)
    try:
        logging.info('starting the server (port: ' + str(port) + ')')
        enocean.receive(background=True)
        server.start()
    except KeyboardInterrupt:
        logging.info('stopping the server')
        server.stop()
        enocean.stop()
        logging.info('done')
        return
    except Exception as e:
        logging.error(e)
        sleep(3)
